import { world, Player, EntityEquippableComponent, EquipmentSlot, BlockPermutation, EntityItemComponent, ItemStack } from '@minecraft/server';
import { autoSmeltItems, AutoSmeltRawOres, SmeltedConversions } from './lib/constants';

world.afterEvents.itemUseOn.subscribe((event) => {
  const { itemStack, source, block } = event;

  if (!(source instanceof Player) || !block.hasTag('dirt') || itemStack.typeId !== 'heavy:magma_hoe') return;
  block.setPermutation(BlockPermutation.resolve('dirt', { dirt_type: 'coarse' }));
});

world.afterEvents.playerBreakBlock.subscribe((event) => {
  const { block, dimension, brokenBlockPermutation, itemStackAfterBreak: itemStack } = event;
  const { location } = block;

  if (!autoSmeltItems.includes(itemStack?.typeId)) return;
  if (!Object.keys(SmeltedConversions).some((blockTypeId) => brokenBlockPermutation.matches(blockTypeId))) return;
  const itemEntities = dimension.getEntities({ type: 'minecraft:item', maxDistance: 1.9, location });

  if (itemEntities.length < 1) return;
  world.playSound('random.fizz', location);
  itemEntities.map((itemEntity) => {
    const itemStackComponent = itemEntity.getComponent(EntityItemComponent.componentId);

    if (!itemStackComponent) return;
    const { itemStack } = itemStackComponent;
    const conversion = AutoSmeltRawOres[itemStack.typeId] || SmeltedConversions[itemStack.typeId];

    if (!conversion) return;
    itemEntity.remove();
    dimension.spawnItem(new ItemStack(conversion), location);
  });
});

world.afterEvents.entityHurt.subscribe((event) => {
  const { hurtEntity, damageSource } = event;
  const { damagingEntity: player } = damageSource;

  if (!(player instanceof Player)) return;
  const entityEquipmentComponent = player.getComponent(EntityEquippableComponent.componentId);
  const mainHand = entityEquipmentComponent.getEquipment(EquipmentSlot.Mainhand);

  if (!mainHand || mainHand?.typeId !== 'heavy:magma_sword') return;
  hurtEntity?.setOnFire(8, true);
});
